﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Diagnostics;
using System.Collections.Specialized;

namespace System.Collections.Observable.DebugViews
{
    internal class ObservableCollectionBaseDebugView<TValue, TCollection> where TCollection : ICollection<TValue>
    {
        private ObservableCollectionBase<TValue, TCollection> ObservableCollection;

        [DebuggerBrowsable(DebuggerBrowsableState.Collapsed)]
        [DebuggerDisplay("{InternalCollection.Count}", Name = "Internal Collection")]
        public TCollection InternalCollection
        {
            get
            {
                return ObservableCollection.InternalCollection;
            }
        }

        [DebuggerDisplay("{SubscribersCount}")]        
        public NotifyCollectionChangedEventHandler Subscribers
        {
            get
            {
                return ObservableCollection.CollectionChangedSubscribers;
            }
        }

        private int SubscribersCount
        {
            get
            {   
                NotifyCollectionChangedEventHandler subscribers = ObservableCollection.CollectionChangedSubscribers;
                if (subscribers == null)
                {
                    return 0;
                }
                else
                {
                    return subscribers.GetInvocationList().Length;
                }
            }
        }

        public ObservableCollectionBaseDebugView(ObservableCollectionBase<TValue, TCollection> observableCollection)
        {
            ObservableCollection = observableCollection;
        }
    }
}
